@@ -6,7 +6,7 @@ use vars qw(@ISA $VERSION %CIPHERS);
 require DynaLoader;
 
 @ISA = qw(DynaLoader);
-$VERSION = '0.22';
+$VERSION = '0.25';
 
 bootstrap Crypt::SSLeay $VERSION;
 
@@ -59,6 +59,10 @@ __END__
   # DEFAULT SSL VERSION
   $ENV{HTTPS_VERSION} = '3';
 
+  # CLIENT CERT SUPPORT
+  $ENV{HTTPS_CERT_FILE} = 'certs/notacacert.pem';
+  $ENV{HTTPS_KEY_FILE}  = 'certs/notacakeynopass.pem';
+
 =head1 DESCRIPTION
 
 This perl module provides support for the https
@@ -155,6 +159,29 @@ this way:
   $ENV{HTTPS_PROXY_USERNAME} = 'username';
   $ENV{HTTPS_PROXY_PASSWORD} = 'password';  
 
+=head1 CLIENT CERTIFICATE SUPPORT
+
+Certificate support is new, provided by patches from Tobias Manthey.
+
+PEM encoded certificate and private key files may be used like this:
+
+  $ENV{HTTPS_CERT_FILE} = 'certs/notacacert.pem';
+  $ENV{HTTPS_KEY_FILE}  = 'certs/notacakeynopass.pem';
+
+You may test your files with the ./net_ssl_test program
+by issuing a command like:
+
+  ./net_ssl_test -cert=certs/notacacert.pem -key=certs/notacakeynopass.pem -d GET $HOST_NAME
+
+=head2 Creating a Test Certificate
+
+To create simple test certificates with openssl, you may:
+
+     /usr/local/openssl/bin/openssl req -config /usr/local/openssl/openssl.cnf -new -days 365 -newkey rsa:1024 -x509 -keyout notacakey.pem -out notacacert.pem 
+
+To remove the pass phrase from the key file, execute this:
+     /usr/local/openssl/bin/openssl rsa -in notacakey.pem -out notacakeynopass.pem
+
 =head1 SSL VERSIONS
 
 Crypt::SSLeay tries very hard to connect to ANY SSL web server
@@ -182,7 +209,8 @@ connection attempt order remains unchanged by this setting.
 
  PLATFORM	CPU 	SSL		PERL	 VER	DATE		WHO
  --------	--- 	---		----	 ---	----		---
- Linux 2.2.14 	x86	OpenSSL 0.9.6	5.00503	 .21	2001-01-10	Joshua Chamas
+ Solaris 2.7    SPARC   OpenSSL 0.9.6   5.00503  .22    2001-03-01      Dave Paris
+ Linux 2.2.14   x86	OpenSSL 0.9.6	5.00503	 .21	2001-01-10	Joshua Chamas
  WinNT SP6 	x86	OpenSSL 0.9.4	5.00404	 .21	2001-01-10	Joshua Chamas
  AIX 4.3.2	RS/6000	OpenSSL 0.9.6	5.6.0	 .19	2001-01-08	Peter Heimann
  Solaris 2.6	x86	OpenSSL 0.9.5a	5.00501	 .17    2000-09-04	Joshua Chamas
@@ -260,6 +288,11 @@ leak that has been the bane of many a Crypt::SSLeay user.
 Thanks to Bryan Hart for his patch adding proxy support,
 and thanks to Tobias Manthey for submitting another approach.
 
+Thanks to Alex Rhomberg for Alpha linux ccc patch.
+
+Thanks to Tobias Manthey for his patches for client 
+certificate support.
+
 =head1 SUPPORT
 
 For OpenSSL and Crypt::SSLeay support, please email the 
